<?php
/**
 * Jamroom 5 Search module
 *
 * copyright 2003 - 2015
 * by The Jamroom Network
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0.  Please see the included "license.html" file.
 *
 * This module may include works that are not developed by
 * The Jamroom Network
 * and are used under license - any licenses are included and
 * can be found in the "contrib" directory within this module.
 *
 * Jamroom may use modules and skins that are licensed by third party
 * developers, and licensed under a different license  - please
 * reference the individual module or skin license that is included
 * with your installation.
 *
 * This software is provided "as is" and any express or implied
 * warranties, including, but not limited to, the implied warranties
 * of merchantability and fitness for a particular purpose are
 * disclaimed.  In no event shall the Jamroom Network be liable for
 * any direct, indirect, incidental, special, exemplary or
 * consequential damages (including but not limited to, procurement
 * of substitute goods or services; loss of use, data or profits;
 * or business interruption) however caused and on any theory of
 * liability, whether in contract, strict liability, or tort
 * (including negligence or otherwise) arising from the use of this
 * software, even if advised of the possibility of such damage.
 * Some jurisdictions may not allow disclaimers of implied warranties
 * and certain statements in the above disclaimer may not apply to
 * you as regards implied warranties; the other terms and conditions
 * remain enforceable notwithstanding. In some jurisdictions it is
 * not permitted to limit liability and therefore such limitations
 * may not apply to you.
 *
 * @copyright 2012 Talldude Networks, LLC.
 */

// make sure we are not being called directly
defined('APP_DIR') or exit();

/**
 * config
 */
function jrSearch_config()
{
    // Additional Search Fields
    $_tmp = array(
        'name'     => 'search_fields',
        'type'     => 'textarea',
        'default'  => '',
        'validate' => 'printable',
        'label'    => 'additional search fields',
        'help'     => 'If you would like to have additional DataStore fields available for search, enter the DataStore field name, one per line.<br><br><strong>Example:</strong><br>If you have created a custom User Profiles field via the Form Designer called &quot;profile_location&quot; you would enter <strong>profile_location</strong> on a line by itself to enable that field to be searched.',
        'section'  => 'search options',
        'order'    => 1
    );
    jrCore_register_setting('jrSearch', $_tmp);

    // Additional Search Fields
    $_tmp = array(
        'name'     => 'fulltext',
        'type'     => 'checkbox',
        'default'  => 'off',
        'validate' => 'onoff',
        'label'    => 'enable full text search',
        'help'     => 'Enabling this option will activate a special <strong>Full Text</strong> search index for modules - a Full Text search can do a better job of finding relevant items when performing a search of textual data.',
        'section'  => 'search options',
        'order'    => 2
    );
    jrCore_register_setting('jrSearch', $_tmp);

    // Search index limit
    $_tmp = array(
        'name'     => 'index_limit',
        'type'     => 'text',
        'default'  => '4',
        'validate' => 'number_nz',
        'label'    => 'index result count',
        'help'     => 'How many results from each module that has matching results should be shown on the results page?',
        'section'  => 'search options',
        'order'    => 3
    );
    jrCore_register_setting('jrSearch', $_tmp);

    // Search index limit
    $_tmp = array(
        'name'     => 'result_limit',
        'type'     => 'text',
        'default'  => '12',
        'validate' => 'number_nz',
        'label'    => 'module result count',
        'help'     => 'When viewing the search results for an individual module, how many results should be shown on each page?',
        'section'  => 'search options',
        'order'    => 4
    );
    jrCore_register_setting('jrSearch', $_tmp);

    return true;
}

/**
 * Make sure full text indexing is not in progress
 * @param $_post array Posted data
 * @param $_user array Viewing user info
 * @param $_conf array Global Config
 * @return bool
 */
function jrSearch_config_display($_post, $_user, $_conf)
{
    // See if we have an index going on
    $_tm = jrCore_get_active_queue_info();
    if (isset($_tm['jrSearch']['search_index'])) {
        jrCore_set_form_notice('error', 'Full Text Indexing is currently in progress<br>Wait for this message to disappear before making additional changes', false);
    }
    return true;
}

/**
 * Validate the posted config info
 * @param $_post array Posted data
 * @return mixed
 */
function jrSearch_config_validate($_post)
{
    global $_conf;
    // Get our EXISTING setup
    if (isset($_conf['jrSearch_fulltext']) && $_conf['jrSearch_fulltext'] == 'off' && $_post['fulltext'] == 'on') {

        // We are being turned on - create new indexes
        $_md = jrCore_get_datastore_modules();
        if ($_md && is_array($_md)) {
            foreach ($_md as $mod => $pfx) {
                // Some module add items to OTHER modules - exclude those here
                switch ($mod) {
                    case 'jrTags':
                    case 'jrRating':
                        continue 2;
                        break;
                }
                $_queue = array(
                    'action' => 'create',
                    'module' => $mod
                );
                jrCore_queue_create('jrSearch', 'search_index', $_queue);
            }
        }
    }
    return $_post;
}
